/***************************************************************************
 *   Copyright (C) 2004 - 2005 by Raphael Langerhorst                      *
 *   raphael-langerhorst@gmx.at                                            *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/

#ifndef GWEADVANCEDCONTROLLER_H
#define GWEADVANCEDCONTROLLER_H

#include "GweController.h"

#include <qthread.h>

namespace GWE
{

class GDataController;

/**
 * \Class GweAdvancedController GweAdvancedController.h
 * \brief An advanced GWE Controller with networking and persistent storage.
 * @author Raphael Langerhorst
 */
class GweAdvancedController : public GweController, protected QThread
{
  Q_OBJECT

  private:

    /**
     * Set to true by interruptThread(). If this
     * flag is set, the run() method stops execution.
     */
    bool Interrupted;
  
  public:
  
    /**
     * Constructor.
     */
    GweAdvancedController(GDataController* data, QObject *parent = 0, const char *name = 0);

    /**
     * Virtual Destructor. Also stops the thread and waits() for it.
     */
    virtual ~GweAdvancedController();

  protected:

    /**
     * Periodically forces saving of open elements to the storage.
     *
     * Basically this will also force element data propagation to
     * other servers.
     *
     * Stops when Interrupted is set to true.
     */
    void run();

  protected slots:

    /**
     * Sets Interrupted to true.
     */
    void interruptThread();
    
    /**
     * Starts the thread execution of this element and returns.
     *
     * A single shot timer is started in the constructor that
     * uses this method as slot.
     * 
     * @see run()
     */
    void startControllerThread();
    
};

}

#endif //GWEADVANCEDCONTROLLER_H

